#/usr/bin/perl
<<DOC; 
Nom : Rachid Belmouhoub
Avril 2012
 usage : perl bao3_rb_new.pl fichier_tag fichier_motif
DOC

use strict;
use utf8;
use XML::LibXML;

# Dfinition globale des encodage d'entre et sortie du script  utf8
binmode STDIN,  ':encoding(utf8)';
binmode STDOUT, ':encoding(utf8)';

# On vrifie le nombre d'arguments de l'appel au script ($0 : le nom du script)
if($#ARGV!=1){print "usage : perl $0 fichier_tag fichier_motif";exit;}

# Enregistrement des arguments de la ligne de commande dans les variables idoines
my $tag_file= shift @ARGV;
my $patterns_file = shift @ARGV;

# cration de l'objet XML::XPath pour explorer le fichier de sortie tree-tagger XML
my $xp = XML::LibXML->new(XML_LIBXML_RECOVER => 2);
$xp->recover_silently(1);

my $dom    = $xp->load_xml( location => $tag_file );
my $root   = $dom->getDocumentElement();

my $xpc    = XML::LibXML::XPathContext->new($root);

# Ouverture du fichiers de motifs
open(PATTERNSFILE, $patterns_file) or die "can't open $patterns_file: $!\n";

# lecture du fichier contenant les motifs, un motif par ligne (par exemple : NOM ADJ)
while (my $ligne = <PATTERNSFILE>) {
	# Appel   la procdure d'extraction des motifs
	&extract_pattern($ligne);
}

# Fermeture du fichiers de motifs
close(PATTERNSFILE);

# routine de construction des chemins XPath
sub construit_XPath{
	# On rcupre la ligne du motif recherch
	my $local_ligne=shift @_;
	
	# initialisation du chemin XPath
	my $search_path="";
	
	# on supprime avec la fonction chomp un ventuel retour  la ligne
	chomp($local_ligne);
	
	# on limine un veltuel retour chariot hrit de windows
	$local_ligne=~ s/\r$//;
	
	# Construction au moyen de la fonction split d'un tableau dont chaque lment a pour valeur un lment du motif recherch
	my @tokens=split(/ /,$local_ligne);
	
	# On commence ici la construction du chemin XPath
	# Ce chemin correspond au premier noeud "element" de l'arbre XML qui rpond au motif cherch 
	$search_path="//element[contains(data[\@type=\"type\"],\"$tokens[0]\")]";
	
	# Initialisation du compteur pour la boucle de construction du chemin XPath
	my $i=1;
	while ($i < $#tokens) {
		$search_path.="[following-sibling::element[1][contains(data[\@type=\"type\"],\"$tokens[$i]\")]";
		$i++;
	}
	my $search_path_suffix="]";
	
	# on utilise l'oprateur x qui permet de rpter la chaine de caractre  sa gauche autant de fois que l'entier  sa droite,
	# soit $i fois $search_path_suffix
	$search_path_suffix=$search_path_suffix x $i;
	
	# le chemin XPath final
	$search_path.="[following-sibling::element[1][contains(data[\@type=\"type\"],\"".$tokens[$#tokens]."\")]"
								.$search_path_suffix;
		# print  "$search_path\n";

	# on renvoie  la procdure appelante le chein XPath et le tableau des lments du motif
	return ($search_path,@tokens);
}

# routine d'extraction du motif
sub extract_pattern{
	# On rcupre la ligne du motif recherch
	my $ext_pat_ligne= shift @_;

	# Appel de la fonction construit_XPath pour le motif lu  la ligne courrante du fichier de motif
	my ($search_path,@tokens) = &construit_XPath($ext_pat_ligne);
	
	# dfinition du nom du fichier de rsultats pour le motif en utilisant la fonction join
	my $match_file = "res_extract-".join('_', @tokens).".txt";

	# Ouverture du fichiers de rsultats encod en UTF-8
	open(MATCHFILE,">:encoding(UTF-8)", "$match_file") or die "can't open $match_file: $!\n";
	
	# cration de l'objet XML::XPath pour explorer le fichier de sortie tree-tagger XML
	
	# Parcours des noeuds du ficher XML correspondant au motif, au moyen de la mthode findnodes
	# qui prend pour argument le chemin XPath construit prcdement
	# avec la fonction "construit_XPath"
	my @nodes=$root->findnodes($search_path);
	foreach my $noeud ( @nodes) {
		# Initialisation du chemin XPath relatif du noeud "data" contenant
		# la forme correspondant au premier lment du motif
		# Ce chemin est relatif au premier noeud "element" du bloc retourn
		# et pointe sur le troisime noeud "data" fils du noeud "element"
		# en l'identifiant par la valeur "string" de son attribut "type"
		my $form_xpath="";
		$form_xpath="./data[\@type=\"string\"]";
		
		# Initialisation du compteur pour la boucle d'xtraction des formes correspondants
		# aux lments suivants du motif
		my $following=0;

		# Recherche du noeud data contenant la forme correspondant au premier lment du motif		
		# au moyen de la fonction "find" qui prend pour arguments:
		#			1. le chemin XPath relatif du noeud "data"
		#			2. le noeud en cours de traitement dans cette boucle foreach
		# la fonction "find" retourne par dfaut une liste de noeuds, dans notre cas cette liste
		# ne contient qu'un seul lment que nous rcuprons avec la fonction "get_node"
		# enfin nous en imprimons le contenu textuel au moyen de la mthode string_value
		print MATCHFILE $xpc->findvalue($form_xpath,$noeud);
		
		# Boucle d'xtraction des formes correspondants aux lments suivants du motif
		# On descend dans chaque noeud element du bloc
		while ( $following < $#tokens) {
			# Incrmentation du compteur $following de cette boucle d'xtraction des formes
			$following++;
			
			# Construction du chemin XPath relatif du noeud "data" contenant
			# la forme correspondant  l'lment suivant du motif
			# Notez bien l'utilisation du compteur $folowing tant dans la condition de la boucle ci-dessus
			# que dans la construction du chemin relatif XPath
			my $following_elmt="following-sibling::element[".$following."]";			
			$form_xpath=$following_elmt."/data[\@type=\"string\"]";

			#	Impression du contenu textuel du noeud data contenant la forme correspondant  l'lment suivant du motif
			print MATCHFILE " ",$xpc->findvalue($form_xpath,$noeud);
		
			# Incrmentation du compteur $following de cette boucle d'xtraction des formes
			# $following++;
		}
		print MATCHFILE "\n";
	}
	# Fermeture du fichiers de motifs
	close(MATCHFILE);
}
